@@ -96,7 +96,7 @@ struct CompatSet {
 	   p != names.end();
 	   ++p) {
 	char s[10];
-	snprintf(s, sizeof(s), "%lld", (unsigned long long)p->first);
+	snprintf(s, sizeof(s), "feature_%lld", (unsigned long long)p->first);
 	f->dump_string(s, p->second);
       }
     }
