@@ -34,7 +34,7 @@ typename T::iterator rand_choose(T &cont) {
   int index = rand() % cont.size();
   typename T::iterator retval = cont.begin();
 
-  for (; index > 0; --index) retval++;
+  for (; index > 0; --index) ++retval;
   return retval;
 }
 
@@ -139,6 +139,7 @@ public:
   map<int, map<string,ObjectDesc> > pool_obj_cont;
   set<string> oid_in_use;
   set<string> oid_not_in_use;
+  set<int> snaps_in_use;
   int current_snap;
   string pool_name;
   librados::IoCtx io_ctx;
@@ -215,7 +216,7 @@ public:
     state_lock.Lock();
 
     TestOp *next = gen->next(*this);
-    while (next || inflight.size()) {
+    while (next || !inflight.empty()) {
       if (next) {
 	inflight.push_back(next);
       }
@@ -235,7 +236,7 @@ public:
 	  }
 	}
 	
-	if (inflight.size() >= (unsigned) max_in_flight || (!next && inflight.size())) {
+	if (inflight.size() >= (unsigned) max_in_flight || (!next && !inflight.empty())) {
 	  cout << "Waiting on " << inflight.size() << std::endl;
 	  wait();
 	} else {
@@ -499,7 +500,7 @@ public:
 	    op.rmxattr(i->first.c_str());
 	  }
 	}
-	if (!to_remove.size()) {
+	if (to_remove.empty()) {
 	  context->kick();
 	  context->oid_in_use.erase(oid);
 	  context->oid_not_in_use.insert(oid);
@@ -563,7 +564,6 @@ public:
   void _begin()
   {
     ContDesc cont;
-    stringstream acc;
     {
       Mutex::Locker l(context->state_lock);
       cont = ContDesc(context->seq_num, context->current_snap,
@@ -656,7 +656,6 @@ public:
   void _begin()
   {
     ContDesc cont;
-    stringstream acc;
     {
       Mutex::Locker l(context->state_lock);
       cont = ContDesc(context->seq_num, context->current_snap,
@@ -877,8 +876,6 @@ public:
       return;
     }
 
-    stringstream acc;
-
     ObjectDesc contents(&context->cont_gen);
     context->find_object(oid, &contents);
     bool present = !contents.deleted();
@@ -1302,6 +1299,8 @@ public:
     }
     context->oid_in_use.insert(oid);
     context->oid_not_in_use.erase(oid);
+    context->snaps_in_use.insert(roll_back_to);
+
     context->roll_back(oid, roll_back_to);
     uint64_t snap = context->snaps[roll_back_to];
 
@@ -1327,7 +1326,8 @@ public:
       Mutex::Locker l(context->state_lock);
       context->oid_in_use.erase(oid);
       context->oid_not_in_use.insert(oid);
-    }
+      context->snaps_in_use.erase(roll_back_to);
+   }
   }
 
   string getType()
